<?php

namespace Drupal\bootstrap_theme_toggler\Plugin\Block;

use Drupal\Core\Block\BlockBase;
use Drupal\Core\Form\FormStateInterface;


/**
 * Provides a 'Theme Toggler' Block.
 *
 * @Block(
 *   id = "theme_toggler",
 *   admin_label = @Translation("Theme Toggler"),
 *   category = @Translation("Theme Toggler Block"),
 * )
 * 
 * When place, the machine name of the block instance will be enforced to be
 * the default name provided by Drupal, which should be the Block 'id' above,
 * without the underscore, 'themetoggler', prefixed by the default theme name
 * resulting in a machine name of 'loaded_default_theme_themetoggler'.
 * This is used to enforce only 1 instance of this block is placed on the site.
 */
class ThemeToggler extends BlockBase {

  /**
   * {@inheritdoc}
   */
  public function build() {
    $config = $this->configuration;

    if (!$config['custom_only']) {
      // array of default bootstrap theme modes, use all or none.
      $bm = array("auto"=>"auto", "light"=>"light", "dark"=>"dark");
    } else {
      $bm = array();
    }
    // Need to gaurd against null, color modes, may not exist.
    if (!empty($config['color_modes'])) {
      if (!empty($bm)) {
        $themeModes = array_merge($bm, $config['color_modes']);
      } else {
        $themeModes = $config['color_modes'];
      }
    } else {
      $themeModes = $bm;
    }
    return [
      '#theme' => 'block__toggler',
      '#attached' => [
        'library' => ['bootstrap_theme_toggler/theme-toggler'],
      ],
      '#toggler_label' => $config['toggler_label'],
      '#show_toggler_label' => $config['show_toggler_label'],
      '#show_option_labels' => $config['show_option_labels'],
      '#show_icons' => $config['show_icons'],
      '#show_checkmark' => $config['show_checkmark'],
      '#theme_modes' => $themeModes,
      // '#custom_only' => $config['custom_only'],
    ];
  }


  /**
   * Returns generic default configuration for block plugins.
   *
   * @return array
   *   An associative array with the default configuration.
   */
  protected function baseConfigurationDefaults() {
    return [
      'label' => 'Bootstrap Theme Toggler',
      'label_display' => 0,
      'provider' => $this->pluginDefinition['provider'],
      //   '#default_value' => false,
      //   '#return_value' => false,
    ];
  }


  /**
   * {@inheritdoc}
   */
  public function blockForm($form, FormStateInterface $form_state) {

    $form['toggler_label'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Toggler Label'),
      '#description' => $this->t('Text to display on the toggler dropdown button.'),
      '#default_value' => $this->configuration['toggler_label'] ?? 'Theme',
    ];
    $form['show_toggler_label'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Show toggler label'),
      // '#description' => $this->t('Check to make button label visible, on Toggler.'),
      '#default_value' => $this->configuration['show_toggler_label'] ?? true,
    ];
    $form['show_option_labels'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Show theme mode option labels in dropdown list.'),
      // '#description' => $this->t('Check to make drop-down option labels visible, on Toggler.'),
      '#default_value' => $this->configuration['show_option_labels'] ?? true,
    ];
    $form['show_icons'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Show toggler color mode icons'),
      '#description' => $this->t("To change icons being displayed, edit/replace the appropriate svg with the name of the theme mode to update. The svg element inside must be defined by a &lt;sybmol&gt; element, with an id matching the theme name. The filename should be the theme name prefixed by 'icon_'. <br>A replacement icon for the 'light' theme mode should have the tag, <code><xmp><symbol id='light'><path>...</path></symbol></xmp></code>and have the filename and location of <b>`bootstrap_theme_toggler/svg/icon_light.svg`</b>."),
      '#default_value' => $this->configuration['show_icons'] ?? true,
    ];
    $form['show_checkmark'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Show a checkmark next to active theme'),
      // '#description' => $this->t('Check to show a checkmark next to active theme.'),
      '#default_value' => $this->configuration['show_checkmark'] ?? true,
    ];

    // DO NOT USE - the approach below, as this may retreive the admin theme
    // instead of the desired general site theme, which is the 'default' theme.
    // $themePath = \Drupal::service('theme.manager')->getActiveTheme()->getPath();

    // Get the the loaded 'default' theme, not the 'active' theme of the page.
    $themeRoot = \Drupal::config('system.theme')->getOriginal('default');
    $defaultTheme = \Drupal::service('extension.list.theme')->getPath($themeRoot).'/scss/';

    $prefix = 'theme_';
    $result = array();
    $defaultValues = array();

    // Look for any scss files inside the 'default_theme_root/scss' folder,
    // that has the prefix 'theme_'. It is assumed that if this folder exists,
    // that the theme is a bootstrap SASS based theme.
    // To add additional theme modes, create an appropriate scss file with the
    // name 'theme_modename' where, 'modename', is a lowercase theme mode ID.
    // EX: for a mode called 'blue', add 'theme_blue.scss' to the scss folder.
    $handle = @opendir($defaultTheme);
    if (!empty($handle)) {
      while ($entry = @readdir($handle)) {
        // If item is a file and has matching prefix, process as a theme mode.
        if(is_file($defaultTheme.$entry) && (mb_substr($entry, 0, 6) == $prefix)) {
          $result[str_replace($prefix, '', pathinfo($entry, PATHINFO_FILENAME))] = str_replace($prefix, '', pathinfo($entry, PATHINFO_FILENAME));
          // Building array of values only, for use in setting default_value.
          $defaultValues[] = str_replace($prefix, '', pathinfo($entry, PATHINFO_FILENAME));
        }
      }
      closedir($handle);
    }

    // Creates checkbox to enable/disable each custom scss theme modes found.
    if (!empty($result)){
      // $config = $this->configuration['color_modes'] ?? $defaultValues;
      $config = $this->configuration['color_modes'] ?? ($defaultValues ?? null);
      // For checkboxes, the array keys are used to set the checkbox values
      // and the array values are used to set the checkbox labels.
      $form['color_modes'] = array(
        '#type' => 'checkboxes',
        '#title' => $this->t('Custom Theme Modes Available'),
        '#options' => $result,
        '#description' => $this->t('Select custom theme modes to include on toggler toggler.'),
        '#default_value' => $config,
      );
      $form['custom_only'] = [
        '#type' => 'checkbox',
        '#title' => $this->t('Use only custom theme modes'),
        '#description' => $this->t("Checking this will disable 'light', 'dark' and 'auto' themes."),
        '#default_value' => $this->configuration['custom_only'] ?? false,
      ];
    } else {
      $form['text_no_colors'] = array(
        '#type' => 'item',
        '#title' => $this->t('No Custom Theme Modes Found'),
        '#description' => $this->t("
        Custom Theme Modes are only available for bootstrap SASS based themes.<br>
        To create custom theme modes, define a color mode, theme palette `<b>[data-bs-theme='nameid'] {  styles  }`</b> in an scss file, place it inside the <b>`theme_root/scss/`</b> folder, then add new file to the bottom of <b>`import.scss`</b>.<br>
        The theme file with should be named using the prefix, <b>'theme_'</b>, following the convention: <b>'theme_nameid'</b><br>
        EX: a theme called <b>'blue'</b>, should have a theme definition, <b>[data-bs-theme='blue'] { styles }</b>, and have the filename <b>'theme_blue.scss'</b>
        "),
      );
    }

    return $form;
  }
  // END blockForm


  /**
   * {@inheritdoc}
   */
  public function blockValidate ($form, FormStateInterface $form_state) {
    
    // Check that the machine name matches the required default name.
    // Used to prevent more than one instance of this block being placed.

    // Get default theme name used to determine the block's machine name.
    $defaultTheme = \Drupal::config('system.theme')->getOriginal('default');
    // Get the actual machine name set for current block instance.
    $machine_name = $form_state->get('machine_name.initial_values');
    $defaultName = $machine_name['id'] == $defaultTheme.'_themetoggler';

    if (!$defaultName) {
      $form_state->setErrorByName('machine_name', $this->
          t("Another instance of this block already exists, or the default machine name has been changed. <br> Make sure toggler machine name is 'DEFAULT_THEME_themetoggler', and that it has not already been placed into your site."));
    }
    $values = $form_state->getValues();
    if (!empty($values['custom_only'])) {
      if (empty(array_filter( $values['color_modes'] ?? [0] ))){
        $form_state->setErrorByName('color_modes', $this->
          t("Cannot select 'Custom Only' and have no custom themes enabled."));
      }
    }
  }


  /**
   * {@inheritdoc}
   */
  public function blockSubmit($form, FormStateInterface $form_state) {

    $values = $form_state->getValues();
    $this->configuration['toggler_label'] = $values['toggler_label'];
    $this->configuration['show_toggler_label'] = $values['show_toggler_label'];
    $this->configuration['show_option_labels'] = $values['show_option_labels'];
    $this->configuration['show_icons'] = $values['show_icons'];
    $this->configuration['show_checkmark'] = $values['show_checkmark'];
    $this->configuration['custom_only'] = $values['custom_only'] ?? null;
    $this->configuration['color_modes'] = $values['color_modes'] ?? false;
    
  }

}
// END ThemeToggler class
